USE [EVX]
GO


CREATE PROCEDURE [evx].[ProcessTargetQueue]
AS
SET NOCOUNT ON;

DECLARE 
 @conversation_handle uniqueidentifier,
 @service_name nvarchar(512),
 @service_contract_name nvarchar(256),
 @message_type_name nvarchar(256),
 @validation nchar(2),
 @message_body varbinary(max)
 
 DECLARE @StartTime DATEtIME = GETDATE(),
		@MessageCount INT = 0;

--
-- Notice no transaction
-- 

 BEGIN TRY 
	 WHILE (1=1)
	   BEGIN 
		 WAITFOR(
			RECEIVE TOP(1)
				@service_name = service_name,
				@service_contract_name = service_contract_name,
				@message_body=message_body,
				@message_type_name = message_type_name,
				@validation = validation,
				@conversation_handle = conversation_handle
				FROM evx.TargetQueue
		
		) , timeout 1000;

		IF @@ROWCOUNT = 0
		  BEGIN
			BREAK;
		  END
		
		-- In real life, code right here logs the message.

		if @message_type_name = 'http://schemas.microsoft.com/SQL/Notifications/EventNotification'
		  BEGIN
			EXECUTE [evx].ProcessEvent @message_body;
		  END
		

		if @message_type_name = '//EVX/EndOfStream'
		  begin
			--PRINT 'Ending Stream....';
			END CONVERSATION @conversation_handle;
		  END

		if @message_type_name = 'http://schemas.microsoft.com/SQL/ServiceBroker/EndDialog'
		  BEGIN
			--print 'ENDING CONVERSATION';
			END CONVERSATION @conversation_handle;
		  END


		if @message_type_name = 'http://schemas.microsoft.com/SQL/ServiceBroker/Error'
		  BEGIN
			-- print 'Error...';
	
			INSERT evx.ErrorLog (conversation_handle, service_name, service_contract_name, message_type_name, validation, message_body)
			VALUES (@conversation_handle, @service_name, @service_contract_name, @message_type_name, @validation, @message_body)

			END CONVERSATION @conversation_handle;

		  END


	  END -- WHILE LOOP


END TRY
BEGIN CATCH

	WHILE @@TRANCOUNT > 0 ROLLBACK;
		
	INSERT evx.ErrorLog (conversation_handle, service_name, service_contract_name, message_type_name, validation, message_body,
		[error_line], [error_message], [error_number], [error_procedure], [error_severity], [error_state])
	VALUES (@conversation_handle, @service_name, @service_contract_name, @message_type_name, @validation, @message_body,
				error_line(), error_message(), error_number(), error_procedure(), error_severity(), error_state() )
					
	-- RAISE AN ERROR to fire off the alert
	RAISERROR(50025, 16, 1) with log;

END CATCH

GO


