USE [EVX]
GO

/****** Object:  StoredProcedure [evx].[ProcessTargetQueue]    Script Date: 09/12/2013 13:31:40 ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO




CREATE PROCEDURE [evx].[ProcessTargetQueue]
AS
SET NOCOUNT ON;

DECLARE 
 @conversation_handle uniqueidentifier,
 @service_name nvarchar(512),
 @service_contract_name nvarchar(256),
 @message_type_name nvarchar(256),
 @validation nchar(2),
 @message_body varbinary(max)
 
 DECLARE @StartTime DATEtIME = GETDATE(),
		@MessageCount INT = 0;


 BEGIN TRY 
	 WHILE (1=1)
	   BEGIN 
		 WAITFOR(
			RECEIVE TOP(1)
				@service_name = service_name,
				@service_contract_name = service_contract_name,
				@message_body=message_body,
				@message_type_name = message_type_name,
				@validation = validation,
				@conversation_handle = conversation_handle
				FROM evx.TargetQueue
		
		) , timeout 1000;

		IF @@ROWCOUNT = 0
		  BEGIN
			BREAK;
		  END

		-- First log the message
		if @message_type_name <> '//EVX/EndOfStream'
		  BEGIN
			INSERT evx.MessageLog (conversation_handle, service_name, service_contract_name, message_type_name, validation, message_body)
			VALUES (@conversation_handle, @service_name, @service_contract_name, @message_type_name, @validation, @message_body)
		  END
		  
		-- Increment the message count
		BEGIN TRY

			DECLARE @MessageTime DATETIME = GETDATE();
			SELECT @MessageTime = CAST(CAST(@MessageTime AS VARCHAR(100)) AS DATETIME);

			IF EXISTS (SELECT * FROM evx.MessageCount WHERE MessageTime = @MessageTime)
			  BEGIN
				UPDATE evx.MessageCount
				SET MessageCount = MessageCount +1
				WHERE MessageTime = @MessageTime
			  END	
			ELSE
			  BEGIN
				INSERT evx.MessageCount (MessageTime, MessageCount, QueueSize)
				SELECT
					@MessageTime AS MessageTime,
					1 AS MessageCount,
					(select count(*) from evx.TargetQueue) AS QueueSize
			  END 
			  
			DELETE	evx.MessageCount
			WHERE	MessageTime < DATEADD(d, -90, GETDATE())  

		END TRY
		BEGIN CATCH

			INSERT INTO [evx].[ErrorLog] ([error_line],[error_message],[error_number],[error_procedure],
											[error_severity],[error_state],[LogDate])
			SELECT
				ERROR_LINE(),
				ERROR_MESSAGE(),
				ERROR_NUMBER(),
				ERROR_PROCEDURE(),
				ERROR_SEVERITY(),
				ERROR_STATE(),
				GETDATE()
				
		END CATCH 

		SET @MessageCount += 1;

		if @message_type_name = 'http://schemas.microsoft.com/SQL/Notifications/EventNotification'
		  BEGIN
			EXECUTE [evx].ProcessEvent @message_body;
		  END
		
		--if @message_type_name = '//EVX/TestMessages'
		--  begin
		--	--PRINT 'Test Message....';
		--	--END CONVERSATION @conversation_handle;
		--  END


		if @message_type_name = '//EVX/EndOfStream'
		  begin
			--PRINT 'Ending Stream....';
			END CONVERSATION @conversation_handle;
		  END

		if @message_type_name = 'http://schemas.microsoft.com/SQL/ServiceBroker/EndDialog'
		  BEGIN
			--print 'ENDING CONVERSATION';
			END CONVERSATION @conversation_handle;
		  END


		if @message_type_name = 'http://schemas.microsoft.com/SQL/ServiceBroker/Error'
		  BEGIN
			-- print 'Error...';
	
			INSERT evx.ErrorLog (conversation_handle, service_name, service_contract_name, message_type_name, validation, message_body)
			VALUES (@conversation_handle, @service_name, @service_contract_name, @message_type_name, @validation, @message_body)

			END CONVERSATION @conversation_handle;

		  END


	  END -- WHILE LOOP

	  DELETE evx.MessageLog WHERE LogDate < DATEADD(DAY, -7, GETDATE());
	  
		BEGIN TRY
		INSERT evx.ProcessLog (StartTime, MessageCount, ProcessTime)
		SELECT
			@StartTime AS StartTime,
			@MessageCount AS MessageCount,
			DATEDIFF(ms, @StartTime, GETDATE()) - 1000 AS ProcessTime
			
			DELETE evx.ProcessLog WHERE StartTime < DATEADD(DAY, -90, GETDATE());
			
		END TRY
		BEGIN CATCH
		END CATCH	

END TRY
BEGIN CATCH

	WHILE @@TRANCOUNT > 0 ROLLBACK;
		
	INSERT evx.ErrorLog (conversation_handle, service_name, service_contract_name, message_type_name, validation, message_body,
		[error_line], [error_message], [error_number], [error_procedure], [error_severity], [error_state])
	VALUES (@conversation_handle, @service_name, @service_contract_name, @message_type_name, @validation, @message_body,
				error_line(), error_message(), error_number(), error_procedure(), error_severity(), error_state() )
					
	-- RAISE AN ERROR to fire off the alert
	RAISERROR(50025, 16, 1) with log;

END CATCH




GO


